#include "FastLED.h"  // FastLED library.
#include <SoftwareSerial.h>
#include <DFPlayerMini_Fast.h>

SoftwareSerial mySerial(10, 11); // RX, TX
DFPlayerMini_Fast myMP3;

#if FASTLED_VERSION < 3001000
#error "Requires FastLED 3.1 or later; check github for latest code."
#endif

// Fixed definitions cannot change on the fly.
#define LED_DT 3                                             // Data pin to connect to the strip.
#define LED_TYPE WS2812                                       // Using APA102, WS2812, WS2801. Don't forget to modify LEDS.addLeds to suit.
#define NUM_LEDS 44       //  # of LEDS in the strip
#define COLOR_ORDER GRB  // I had to change this for my strip if your color is off then you know.

// Global variables can be changed on the fly.
uint8_t max_bright = 255;                                      // Overall brightness definition. It can be changed on the fly.

long mycount = 0;

struct CRGB leds[NUM_LEDS];                                   // Initialize our LED array.
const int setpoint = NUM_LEDS * 107;
const int nextLED = 100;     //this is to delay between LEDS
int timer = setpoint + 1000;  //Set the timer beyond what the setpoint is. This will make the LEDs not come on

// constants won't change. They're used here to set pin numbers:
const int buttonPin = 2;    // the number of the pushbutton pin
// Variables will change:
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin
// . int trackOneLeng = 20625;    // This is the length of the first track
bool pulseDone = true;       // Set this to true so that the pulse generator won't trigger


// the following variables are unsigned longs because the time, measured in
// milliseconds, will quickly become a bigger number than can be stored in an int.
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers

int inightlight = 0;    //  Set the nightlight to zero. this will be set to 
int busy = 1; // This means that it is NOT busy.

void setup() {

  Serial.begin(57600);                                        // Initialize serial port for debugging.
  delay(1000);                                                // Soft startup to ease the flow of electrons.
  pinMode(buttonPin, INPUT);
  pinMode(4, INPUT);            //  Reading the playing state of the pin
  
  mySerial.begin(9600);           //Initialize the serial for the DFPlayer Mini
  myMP3.begin(mySerial);

  Serial.println("Setting volume to 10");   //This is being set at 10 since we are using an applified speaker
  myMP3.volume(22);
  delay(2000);    // Delay 1 seconds for the player to register

  LEDS.addLeds<LED_TYPE, LED_DT, COLOR_ORDER>(leds, NUM_LEDS);  // Use this for WS2801 or APA102
//  LEDS.addLeds<LED_TYPE, LED_DT, COLOR_ORDER>(leds, NUM_LEDS);  // Use this for WS2812

  FastLED.setBrightness(max_bright);
  FastLED.clear();
  FastLED.show();
  
  //set_max_power_in_volts_and_milliamps(5, 500);               // FastLED Power management set at 5V, 500mA.
  Serial.println("Setup complete. Starting the loop now.");
} // setup()



void loop(){

  // read the state of the switch into a local variable:
  int reading = digitalRead(buttonPin);
  int busy = digitalRead(4);
  //Serial.print("Button state: ");
  //Serial.println(reading);


  // If the switch changed, due to noise or pressing:

  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }  //if reading lastbuttonstate

   if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:
    // if the button state has changed:
    if (reading != buttonState) {  // . Set the button state to what was last read
      buttonState = reading;
      
      // only toggle the LED if the new button state is HIGH
      if (buttonState == HIGH) {  // . there was a change in the state so do the work      
        if (inightlight == 0) {  //  is nightlight off? yes, start process
          inightlight=1;
          timer = 0;
          //Serial.println("Checking mp3 playing");  //  Check to see if the mp3 is playing
          if(busy == 1){ // not playing an mp3 start the process
            Serial.println("Starting mp3");
            myMP3.play(1);
           }  //if mp3 is playing
         } 
         else { // nightlight is 1 to turn things off by making it zero
          inightlight = 0;
          Serial.println("Turn off the nightlight");
            }  // else nightlight reset        
      }  //if buttonstate
    }  // if reading buttonstate
   }  //if millis is not debouncing


  if (timer <= setpoint){ //  The timer is not reset so don't activate the LEDS.
    setcolors();       
   } 
   

  if (timer >= setpoint && busy == 0 ) { // Check to see if the mp3 is still playing and if so, then pulse.
    if (busy == 0) { // If the sound is still playing then send to pulse
      Pulse();  // Call the pulsing LEDs
    } //  mp3 playing
  }  // if timer and playing mp3
  if (timer >=setpoint && busy == 1) {  // Done with the fade and not playing mp3 anymore, now test for nightlight
    fill_solid(leds, NUM_LEDS, CHSV(171,255,175));  // Set the overall brightness
   }  // if timer done and no playing mp3

   if (inightlight == 0 && busy == 1 ){  // playing of the MP3 is over
       FastLED.clear();  
  }  // if inightlight . 

  FastLED.show();

  
  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastButtonState = reading;  
} // loop()



void setcolors(){
  
  static long mycount = 0;
  if(timer == 0){
    mycount = mycount*0;
    Serial.println("Reset mycount");
    }  // if timer = 0
  
    for (int i=0; i< NUM_LEDS; i++) 
    {
      long temp = mycount / (nextLED*i+1);                    // Each LED starts to light up after the previous one has a brightness of 8. Watch out for div0 though.
      if (temp >0) {             // Once an LED is lit, take the mycount value and subtract the LED location to get the resulting brightness
          temp = temp+mycount-nextLED*i;} 
          temp = constrain(temp,0,255);                     // Keep the absolute value from going over 255
          leds[i] = CHSV(171,255,temp);  
    }//  for loop

  mycount++;
  timer = mycount;
  
} // setcolors()

void Pulse(){ 
   int nbrightness = 0;
   int nfadeAmount = 5;
   int playing = digitalRead(4);  
   do{
      playing = digitalRead(4);  
      for(int i = 0; i < NUM_LEDS; i++ )
       {
       leds[i].setRGB(0,255,250);  // Set Color HERE!!!
       leds[i].fadeLightBy(nbrightness);
      }
      FastLED.show();
      nbrightness = nbrightness + nfadeAmount;
      // reverse the direction of the fading at the ends of the fade: 
      if(nbrightness == 150 || nbrightness == 255)
      {
        nfadeAmount = -nfadeAmount ; 
      }    
      delay(2);  // This delay sets speed of the fade. I usually do from 5-75 but you can always go higher.
      
   } while(playing == 0);
   // . pulseDone = false;  //reset the trigger so this won't get run again
 
}
